@@ -27,6 +27,13 @@ def main_loop():
 def on_new_sound_loaded(controller, graph, sel, curs):
     EditorWindow(controller, graph, sel, curs)
 
+def display_error(title, text, parent=None):
+    d = gtk.MessageDialog(parent, buttons=gtk.BUTTONS_CLOSE)
+    d.set_title(title)
+    d.set_markup(text)
+    d.run()
+    d.destroy()
+
 
 class EditorWindow(gtk.Window):
 
@@ -47,10 +54,20 @@ class EditorWindow(gtk.Window):
         for w in self.toolbar:
             w.set_homogeneous(False)
             w.set_focus_chain([])
+
+        # Create and fill the Effects menu
+        effect_menu = gtk.Menu()
+        for name in controller.list_effects():
+            item = gtk.MenuItem(label=name)
+            item.connect('activate', self.effect, None, name)
+            effect_menu.append(item)
+        w = self.uimanager.get_widget('/menubar/Effects')
+        w.set_submenu(effect_menu)
+
         self.waveform = GraphView(graph, selection, cursor)
         self.scrollbar = GraphScrollbar(graph)
         self.statusbar = gtk.Statusbar()
-        self.filedialog = FileDialog()
+        self.filedialog = FileDialog(controller.list_extensions())
 
         self.vbox = gtk.VBox()
         self.vbox.pack_start(self.menubar, expand=False, fill=False)
@@ -89,6 +106,7 @@ class EditorWindow(gtk.Window):
                 <menuitem action="Cut"/>
                 <menuitem action="Copy"/>
                 <menuitem action="Paste"/>
+                <menuitem action="Mix"/>
                 <separator/>
                 <menuitem action="SelectAll"/>
               </menu>
@@ -98,8 +116,6 @@ class EditorWindow(gtk.Window):
                 <menuitem action="ZoomFit"/>
               </menu>
               <menu action="Effects">
-                <menuitem action="Reverse"/>
-                <menuitem action="Normalize"/>
               </menu>                
               <menu action="Scalpel">
                 <menuitem action="About"/>
@@ -107,7 +123,6 @@ class EditorWindow(gtk.Window):
             </menubar>
             <toolbar name="toolbar">
               <toolitem action="Open"/>
-              <toolitem action="Save"/>
               <separator/>
               <toolitem action="ZoomFit"/>
               <toolitem action="ZoomOut"/>
@@ -123,6 +138,7 @@ class EditorWindow(gtk.Window):
               <toolitem action="Cut"/>
               <toolitem action="Copy"/>
               <toolitem action="Paste"/>
+              <toolitem action="Mix"/>
               <separator/>
               <toolitem action="Undo"/>
               <toolitem action="Redo"/>
@@ -167,6 +183,8 @@ class EditorWindow(gtk.Window):
                    ('Cut', gtk.STOCK_CUT, None, None, '', self.cut),
                    ('Copy', gtk.STOCK_COPY, None, None, '', self.copy),
                    ('Paste', gtk.STOCK_PASTE, None, None, '', self.paste),
+                   ('Mix', gtk.STOCK_ADD, 'Mix', '<Ctrl><Shift>v', None,
+                                                                     self.mix),
                    ('Undo', gtk.STOCK_UNDO, None, '<Ctrl>z', None, self.undo),
                    ('Redo', gtk.STOCK_REDO, None, '<Ctrl><Shift>z', None,
                                                                     self.redo),
@@ -178,8 +196,6 @@ class EditorWindow(gtk.Window):
                                                                 self.zoom_out),
                    ('ZoomIn', gtk.STOCK_ZOOM_IN, None, 'KP_Add', '',
                                                                  self.zoom_in),
-                   ('Reverse', None, 'Reverse', None, '', self.reverse),
-                   ('Normalize', None, 'Normalize', None, '', self.normalize),
                    ('About', gtk.STOCK_ABOUT, None, None, '', self.about)
                    ]
         actiongroup = gtk.ActionGroup('')
@@ -200,11 +216,7 @@ class EditorWindow(gtk.Window):
         self.set_title(title)
 
     def display_error(self, title, text):
-        d = gtk.MessageDialog(parent=self, buttons=gtk.BUTTONS_CLOSE)
-        d.set_title(title)
-        d.set_markup(text)
-        d.run()
-        d.destroy()
+        display_error(title, text, parent=self)
 
     # -- Callbacks
 
@@ -217,9 +229,9 @@ class EditorWindow(gtk.Window):
         """
         if name in ["new", "save", "play", "pause", "rewind", "forward",
                     "goto_start", "goto_end", "select_all",
-                    "cut", "copy", "paste", "undo", "redo",
+                    "cut", "copy", "paste", "mix", "undo", "redo",
                     "zoom_in", "zoom_out", "zoom_fit",
-                    "reverse", "normalize"]:
+                    "effect"]:
             method = getattr(self.ctrl, name)
             def forward(self, *args):
                 method(*args[1:])
@@ -290,6 +302,8 @@ def test():
     win.show_all()
     gtk.main()
 
+    display_error("Title", "Text")
+
 if __name__ == '__main__':
     test()
 else:
