@@ -78,7 +78,7 @@ class LayeredGraphView(LayeredCairoWidget):
 class GraphView(LayeredGraphView):
     """Sound visualization widget for the main window.
 
-    * Two graphical layers: the waveform and the selection.
+    * Three graphical layers: waveform, selection, and cursor.
     * Mouse event listeners act on models (scroll and selection).
 
     """
@@ -86,6 +86,7 @@ class GraphView(LayeredGraphView):
         super(GraphView, self).__init__(graph)
         self._graph = graph
         self._selection = selection
+        self.layers.append(BackgroundLayer(self, selection))
         self.layers.append(WaveformLayer(self, graph))
         self.layers.append(SelectionLayer(self, selection))
         self.layers.append(CursorLayer(self, cursor))
@@ -146,19 +147,56 @@ class WaveformLayer(object):
         "Draw all sound channels."
         if not self._cache:
             surface = context.get_target()
-            self._cache = surface.create_similar(cairo.CONTENT_COLOR,
+            self._cache = surface.create_similar(cairo.CONTENT_COLOR_ALPHA,
                                                  width, height)
             c = cairo.Context(self._cache)
             channels = self._graph.channels()
             numchan = len(channels)
             for i in range(numchan):
-                s = surface.create_similar(cairo.CONTENT_COLOR,
+                s = surface.create_similar(cairo.CONTENT_COLOR_ALPHA,
                                            width, height / numchan)
                 self.draw_channel(channels[i], s, width, height / numchan)
                 c.set_source_surface(s, 0, (height / numchan) * i)
-                c.set_operator(cairo.OPERATOR_ATOP)
+                c.set_operator(cairo.OPERATOR_OVER)
                 c.paint()
                 
+        context.set_source_surface(self._cache, 0, 0)
+        context.set_operator(cairo.OPERATOR_OVER)
+        context.paint()
+
+
+class BackgroundLayer(object):
+    """A layer for LayeredGraphView.
+
+    """
+    def __init__(self, layered, selection):
+        self._layered = layered
+        self._cache = None
+        self._selection = selection
+        self._selection.changed.connect(self.update)
+
+    def update(self):
+        self._cache = None
+        self._layered.redraw()
+
+    def draw(self, context, width, height):
+        if not self._cache:
+            surface = context.get_target()
+            self._cache = surface.create_similar(cairo.CONTENT_COLOR,
+                                                 width, height)
+            c = cairo.Context(self._cache)
+
+            # Black background
+            c.set_source_rgb(0, 0, 0)
+            c.paint()
+
+            # Selection background
+            if self._selection.selected():
+                c.set_source_rgb(0, 0, 0.2)
+                start, end = self._selection.pixels()
+                c.rectangle(start, 0, end - start, height)
+                c.fill()
+            
         context.set_source_surface(self._cache, 0, 0)
         context.set_operator(cairo.OPERATOR_SOURCE)
         context.paint()
@@ -261,6 +299,7 @@ class MouseSelection(object):
 
     """
     def __init__(self, widget, selection):
+        self.widget = widget
         self._selection = selection
         self.pressed = False
         widget.add_events(gtk.gdk.BUTTON_PRESS_MASK |
@@ -270,20 +309,47 @@ class MouseSelection(object):
         widget.connect("button_press_event", self.button_press)
         widget.connect("button_release_event", self.button_release)
         widget.connect("motion_notify_event", self.motion_notify)
-        
+
+    def near(self, x, y):
+        return abs(x - y) < 10
+
     def button_press(self, widget, event):
         if event.button == 1:
             self.pressed = True
-            self._selection.start_selection(event.x)
+            x = event.x
+            start, end = self._selection.pixels()
+            # extend towards left
+            if self._selection.selected() and self.near(start, x):
+                self._selection.pin(end)
+                self._selection.extend(x)
+            # extend towards right
+            elif self._selection.selected() and self.near(end, x):
+                self._selection.pin(start)
+                self._selection.extend(x)
+            # start fresh selection
+            else:
+                self._selection.pin(x)
 
     def motion_notify(self, widget, event):
         if self.pressed:
             x = event.window.get_pointer()[0]
-            self._selection.end_selection(x)
-            
+            self._selection.extend(x)
+
+        self.pointer_style(widget, event)
+
+    def pointer_style(self, widget, event):
+        style = None
+        x = event.window.get_pointer()[0]
+        if self._selection.selected():
+            start, end = self._selection.pixels()
+            if self.near(start, x):
+                style = gtk.gdk.Cursor(gtk.gdk.LEFT_SIDE)
+            elif self.near(end, x):
+                style = gtk.gdk.Cursor(gtk.gdk.RIGHT_SIDE)
+        self.widget.window.set_cursor(style)
+
     def button_release(self, widget, event):
         if event.button == 1:
-            start, end = self._selection.get()
             self.pressed = False
 
 # -- Horizontal scrollbar, subclassed to control a Graph object.
@@ -383,16 +449,22 @@ if __name__ == '__main__':
             layered.layers.append(WaveformLayer(layered, graph))
             return layered
 
-        def selection():
+        def test_selection_layer(layerclass):
             graph = Mock({"channels": [], "set_width": None,
                           "frames_info": (0, 0, 0)})
             graph.changed = Fake()
             selection = Mock({"pixels": (20, 100), "selected": True})
             selection.changed = Fake()
             layered = LayeredGraphView(graph)
-            layered.layers.append(SelectionLayer(layered, selection))
+            layered.layers.append(layerclass(layered, selection))
             return layered
 
+        def background():
+            return test_selection_layer(BackgroundLayer)
+
+        def selection():
+            return test_selection_layer(SelectionLayer)
+
         def cursor():
             graph = Mock({"channels": [], "set_width": None,
                           "frames_info": (0, 0, 0)})
@@ -406,7 +478,8 @@ if __name__ == '__main__':
             layered.layers.append(cursorlayer)
             return layered
 
-        layereds = [randomized(), sine(), sines(), selection(), cursor()]
+        layereds = [randomized(), sine(), sines(), selection(), cursor(),
+                    background()]
 
         for layered in layereds:
             window = gtk.Window()
